/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_GUI_MAIN_WINDOW_HH_
# define WSCOUT_GUI_MAIN_WINDOW_HH_

# include <QtGui/QMainWindow>
# include <QtGui/QLabel>
# include <QtGui/QProgressDialog>
# include <QtGui/QSpinBox>
# include <QtGui/QShortcut>
# include <QtGui/QComboBox>
# include <QtGui/QTabWidget>

# include "wscout_gui_trace.hh"

namespace wscout
{

  namespace gui
  {

    /// WScout's main window.
    class MainWindow : public QMainWindow
    {
      Q_OBJECT

      enum
      {
	status_bar_vmargin	= 1,	///< Status bar's vertical margins.
	status_bar_hmargin	= 4	///< Status bar's horizontal margins.
      };

    public:
      /// Default constructor.
      MainWindow();

      /// Copy constructor.
      MainWindow(const MainWindow&);

    public slots:
      /// Show the open trace window.
      bool		open();

      /// Open a specific trace.
      bool		openTrace(const QString&);

      /// Open a trace in a new window (using an open dialog).
      bool		openInNewWindow() const;

      /// Duplicate this window.
      MainWindow*	duplicateWindow() const;

      /// Open a new empty window.
      MainWindow*	newWindow(bool show = true) const;

      /**
       * Close the currently selected trace.
       *
       * Quit if there is no more opened traces.
       */
      void	closeTab();

      /// Show a FilterDialog instance and filter a trace.
      void	filter();

      /// Change current tab.
      void	stepTab(int);

      /// Switch to previous tab.
      void	prevTab();

      /// Switch to next tab.
      void	nextTab();

      /// Go to the current trace's begining.
      void	begin();


      /// Go to the current trace's end.
      void	end();

      /// Advance into the current trace.
      void	forward();

      /// Step backward into the current trace.
      void	backward();

      /// Show an @c about dialog.
      void	about();

    private slots:
      void	updateTrace();
      void	updatePosition(int);
      void	FinderEditingFinished();

    private:
      struct NewProgressDialog
      {
	NewProgressDialog(QProgressDialog**, QMainWindow*);

	void	operator () (std::streampos, std::streampos);
      private:
	QProgressDialog**	d_;
	QMainWindow*		w_;
      };

      struct UpdateProgressDialog
      {
	UpdateProgressDialog(QProgressDialog**);

	const std::string*	operator () (std::streampos);
      private:
	QProgressDialog**	d_;
      };

      Trace*	trace();

      void			initWidgets();
      void			setupMenuBar();
      void			setupToolBar();
      void			setupCentralWidget();
      void			setupStatusBar();
      static MainWindow*	setupNewWindow(MainWindow* w, bool show);

      QTabWidget		tabs_;
      QLabel			trace_name_;
      QLabel			trace_size_;
      QLabel			position_;
      QComboBox			step_;
      QSpinBox			finder_;
      QShortcut			focus_finder_;
      std::vector<QAction*>	trace_related_actions_;
      std::vector<QAction*>	tabs_related_actions_;
    };

  } // End of namespace gui.

} // End of namespace wscout.

#endif // ! WSCOUT_GUI_MAIN_WINDOW_HH_
