/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <boost/date_time/posix_time/posix_time.hpp>

#include "wscout_gui_pcap_header.hh"

namespace wscout
{

  namespace gui
  {

    PcapHeader::PcapHeader(QWidget* parent): HeaderFrame (parent)
    {
      {
	QFont bold;

	bold.setBold(true);
	id_.setFont(bold);
      }
      {
	QBoxLayout* const l = layout();

	l->addWidget(&id_);
	l->addWidget(&date_);
	l->addStretch();
	l->addWidget(&len_);
      }
    }

    PcapHeader::next_header
    PcapHeader::setup(link_type t, unsigned n, const packet& p)
    {
      using namespace boost::posix_time;
      using namespace boost::gregorian;

      const ptime toa (date(1970, Jan, 01),
		       seconds (p.ts.tv_sec) + microseconds (p.ts.tv_usec));

      setUpdatesEnabled(false);
      id_.setText(tr("#%1").arg(n));
      date_.setText(tr("%1").arg(to_simple_string(toa).c_str()));
      len_.setText(tr("%1 / %2 byte(s)").arg(p.caplen).arg(p.len));
      setUpdatesEnabled(true);

      return nextHeader(t, p);
    }

    PcapHeader::next_header
    PcapHeader::nextHeader(link_type t, const packet& p)
    {
      protocol::id	i;
      switch (t)
	{
	case pcap::Trace::IEEE802_11:
	  i = protocol::IEEE802_11;
	  break;
	case pcap::Trace::PRISM_HEADER:
	  i = protocol::PRISM_HEADER;
	  break;
	default:
	  i = protocol::UNKNOWN;
	  break;
	}
      return next_header (i, p.data.get());
    }

  } // End of namespace gui.

} // End of namespace wscout.
