/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_DUMPER_HH_
# define PCAP_DUMPER_HH_

extern "C"
{
# include <pcap.h>
}

# include <boost/shared_ptr.hpp>

# include <trace-tools/pcap/typedefs.hh>

namespace pcapxx
{

  /**
   * Enable frame/packet dumping to PCAP files.
   *
   * A PCAP %dumper allows you to dump frames/packets into a file.
   * Build a PCAP %dumper from a PCAP descriptor, then use
   * operator()().
   */
  struct dumper
  {
    /**
     * Build a PCAP %dumper from an underlying @c libpcap @c dumper_t.
     *
     * Users should not call this constructor directly.  Create PCAP
     * dumpers using a PCAP descriptor.
     *
     * @see descriptor.
     */
    dumper(pcap_dumper_t*);

    /// Dump a frame/packet given a PCAP header and some data.
    void	operator () (const pkthdr*, const void* pkt);

  private:
    static void	dumper_deleter(pcap_dumper_t*);

    boost::shared_ptr<pcap_dumper_t>	d_;
  };

} // End of namespace pcapxx.

# include "dumper.hxx"

#endif // ! PCAP_DESCRITPOR_HH_
