/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_FRAME_DESCRIPTOR_HXX_
# define PCAP_FRAME_DESCRIPTOR_HXX_

# include <cstring>

# include "frame_descriptor.hh"

namespace pcapxx
{

  inline
  frame_descriptor::frame_descriptor(const pkthdr* h, const void* b):
    pcap_header_ (new pkthdr (*h))
  {
    const size_t	caplen = pcap_header()->caplen;
    uint8_t* const	tmp = new uint8_t [caplen];

    memcpy(tmp, b, caplen);
    bytes_.reset(tmp, delete_uint8_t_array);
  }

  inline
  const frame_descriptor::pkthdr_ptr&
  frame_descriptor::pcap_header() const
  {
    return pcap_header_;
  }

  inline
  const frame_descriptor::bytes_ptr&
  frame_descriptor::bytes() const
  {
    return bytes_;
  }

  inline
  std::ostream&
  frame_descriptor::print(std::ostream& os) const
  {
    return os << pcap_header()->id;
  }

  inline
  bool
  frame_descriptor::operator == (const frame_descriptor& rhs) const
  {
    return
      not memcmp(pcap_header().get(), rhs.pcap_header().get(), sizeof (pkthdr))
      and
      not memcmp(bytes().get(), rhs.bytes().get(), pcap_header()->caplen);
  }

  inline
  bool
  frame_descriptor::operator != (const frame_descriptor& rhs) const
  {
    return not (*this == rhs);
  }

  inline
  void
  frame_descriptor::delete_uint8_t_array(uint8_t* a)
  {
    delete[] a;
  }

  inline
  std::ostream&
  operator << (std::ostream& os, const frame_descriptor& fd)
  {
    return fd.print(os);
  }

} // End of namespace pcapxx.

#endif // ! PCAP_FRAME_DESCRIPTOR_HXX_
