/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PRISM_HEADER_HH_
# define PRISM_HEADER_HH_

/// Prism %header related compounds.
namespace prism
{
  /**
   * @link header Prism %header @endlink %item.
   *
   * @see prism_header
   */
  struct item
  {
    uint32_t did;	///< Unknown.
    uint16_t status;	///< Unkonwn.
    uint16_t len;	///< Length in bytes, should be 4.
    uint32_t data;	///< Item value.
  } ;

  /// Prism %header format.
  struct header
  {
    enum
      {
	/// @link header::devname @c devname @endlink's maximum length.
	devnamelen_max = 16
      };

    uint32_t	msgcode;			///< Unknown.
    uint32_t	msglen;				///< Header length.
    uint8_t	devname[devnamelen_max];	///< Interface name.
    item	hosttime;			///< Host time.
    item	mactime;			///< MAC time.
    item	channel;			///< Channel.
    item	rssi;				///< RSSI.
    item	sq;				///< Unknown.
    item	signal;				///< Signal quality.
    item	noise;				///< Noise information.
    item	rate;				///< Tx/Rx rate.
    item	istx;				///< Unknown.
    item	frmlen;				///< Frame length.
  };

} // End of namespace prism.

#endif // ! PRISM_HEADER_HH_
