/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_LINEAR_REGRESSION_HXX_
# define TOOL_LINEAR_REGRESSION_HXX_

# include <gmpxx.h>

# include "linear_regression.hh"

namespace tool
{

  template <class ImplType, class InputIterator>
  std::pair<ImplType, ImplType>
  linear_regression(const InputIterator& first,
		    const InputIterator& last)
  {
    typename InputIterator::difference_type	n = 0;
    ImplType					x_mean (0);
    ImplType					y_mean (0);

    for (InputIterator i = first; i != last; ++i)
      {
	y_mean += i->first;
	x_mean += i->second;
	++n;
      }
    y_mean /= n;
    x_mean /= n;

    ImplType		cov (0);
    ImplType		V_x (0);

    for (InputIterator i = first; i != last; ++i)
      {
	const ImplType	y_diff = i->first - y_mean;
	const ImplType	x_diff = i->second - x_mean;

	cov += y_diff * x_diff;
	V_x += x_diff * x_diff;
      }
    cov /= n;
    V_x /= n;

    const ImplType	a = cov / V_x;
    const ImplType	b = y_mean - a * x_mean;

    return std::make_pair(a, b);
  }

} // End of namespace tool.

#endif // ! TOOL_LINEAR_REGRESSION_HXX_
