/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_WINDOW_HXX_
# define TOOL_WINDOW_HXX_

# include "window.hxx"

namespace tool
{

  template <class I, unsigned Ws, unsigned S, class B>
  template <class Iterable>
  window<I, Ws, S, B>::window(Iterable& i, bool end):
    at_end_ (end),
    next_ (end ? i.last_ : i.first_),
    last_ (i.last_),
    v_ ()
  {
    for (unsigned i = 0; next_ != last_ and i < Ws; ++i)
      v_.push_back(*next_++);
  }

  template <class I, unsigned Ws, unsigned S, class B>
  bool
  window<I, Ws, S, B>::equal(const window& rhs) const
  {
    return
      at_end_ == rhs.at_end_ and next_ == rhs.next_ and last_ == rhs.last_;
  }

  template <class I, unsigned Ws, unsigned S, class B>
  void
  window<I, Ws, S, B>::increment()
  {
    if (next_ == last_)
      at_end_ = true;
    else
      for (unsigned i = 0; i < S; ++i)
	{
	  v_.pop_front();
	  if (next_ != last_)
	    v_.push_back(*next_++);
	}
  }

  template <class I, unsigned Ws, unsigned S, class B>
  const typename window<I, Ws, S, B>::value_type&
  window<I, Ws, S, B>::get() const
  {
    return v_;
  }

  template <class I, unsigned Ws, unsigned S, class B>
  const typename window<I, Ws, S, B>::value_type*
  window<I, Ws, S, B>::get_ptr() const
  {
    return &v_;
  }

  template <class I, unsigned Ws, unsigned S, class B>
  const typename I::value_type&
  window<I, Ws, S, B>::operator [] (unsigned i) const
  {
    return v_[i];
  }

  template <class I, unsigned Ws, unsigned S, class B>
  window_maker<I, Ws, S, B>::window_maker(const I& first, const I& last):
    first_ (first),
    last_ (last)
  {
  }

} // End of namespace tool.

#endif // ! TOOL_WINDOW_HXX_
