/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_ADDR_MAPPING_HH_
# define WIFI_ADDR_MAPPING_HH_

# include <string>
# include <iostream>
# include <map>

# include <trace-tools/tool/datafile.hh>
# include <trace-tools/wifi/addr.hh>

namespace wifi
{

  /**
   * IEEE 802.11 address to integer mapping designed to be stored into
   * a database file.
   *
   * Load the mapping from the file at creation when the file exists.
   * Store it at deletion so it could be used later, or loaded in
   * a database using a LOAD DATA INFILE SQL command.
   */
  struct addr_mapping : public tool::datafile
  {
    /// Construct the mapping given the database file's name.
    addr_mapping(const std::string&);

    /**
     * Retrieve the index of a corresponding IEEE 802.11 address.
     *
     * When the address exists in the mapping, just return its index.
     * Else, insert the address into the mapping with a new index and
     * return this index.
     */
    unsigned	operator [] (const wifi::addr&);

  protected:
    /// Load a mapping from an @c std::istream.
    void		load(std::istream&);

  private:
    typedef std::map<wifi::addr, unsigned>	map_type;

    map_type	map_;
  };

} // End of namespace wifi.

# include "addr_mapping.hxx"

#endif // ! WIFI_ADDR_MAPPING_HH_
