/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_ADDR_TIMESTAMP_ID_HXX_
# define WIFI_FRAME_ADDR_TIMESTAMP_ID_HXX_

# include "addr_timestamp_id.hh"

# include <trace-tools/wifi/frame/id_helper_hooks.hh>
# include <trace-tools/wifi/frame/dissector/dissector.hh>

namespace wifi
{

  namespace frame
  {

    template <class UniqueId>
    boost::optional<UniqueId>
    make_addr_timestamp_id_from_prism(const pcapxx::frame_descriptor& desc,
				      addr_mapping&	mapping,
				      unsigned		addr_idx,
				      bool		prism_noise_means_error)
    {
      typedef internals::id_helper_hooks	hooks;

      const void*	frame;
      size_t		caplen;

      if (not internals::make_unique_id_handle_prism(frame, caplen, desc,
						     prism_noise_means_error))
	return boost::none_t ();

      const dissector<hooks>	d (frame, caplen);
      const hooks&		h = d.hooks();
      const timestamp*		t = h.timestamp_ptr();
      const addr*		a = h.addr_ptr(addr_idx);

      assert(not t or a);
      if (t)
	return UniqueId (desc.pcap_header()->id, mapping[*a], *t);
      else
	return boost::none_t ();
    }

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_SOURCE_TIMESTAMP_ID_HXX_

