/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_INTERSECTOR_HH_
# define WIFI_FRAME_FILTER_INTERSECTOR_HH_

# include <set>
# include <utility>

# include <trace-tools/tool/microseconds.hh>
# include <trace-tools/tool/iterator_base.hh>

# include "intersector_fwd.hh"

namespace tool
{

  template <class I1, class I2, class B1, class B2>
  struct types< wifi::frame::filter::internals::intersector_iterator<I1, I2,
								     B1, B2> >
  {
    typedef
      std::pair<typename I1::value_type, typename I2::value_type>
      value_type;

    typedef
      wifi::frame::filter::intersector<I1, I2, B1>
      iterable_type;
  };

  template <class I1, class I2, class B>
  struct types< wifi::frame::filter::intersector<I1, I2, B> >
  {
    typedef
      wifi::frame::filter::internals::intersector_iterator<I1, I2, B, bottom>
      iterator;
  };

} // End of namespace tool.

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      namespace internals
      {
	template <class I1, class I2, class B, class Bottom = tool::bottom>
	struct intersector_iterator:
	  tt_inherit(public tool::iterator_base,
		     intersector_iterator<I1, I2, B, Bottom>)
	{
	  typedef tt_get_exact(Bottom, intersector_iterator<I1, I2, B, Bottom>)
	    exact_type;

	  typedef tt_type(iterable_type, exact_type)
	    iterable_type;

	  intersector_iterator(iterable_type& i, bool end);

	  void	increment();

	private:
	  typedef typename iterable_type::arrivals1_type arrivals1_type;
	  typedef typename iterable_type::arrivals2_type arrivals2_type;

	  template <bool SwapResult, class ArrType, class RHSArrType>
	  bool
	  check(I1&		frm,
		ArrType&	frm_arrivals,
		RHSArrType&	rhs_arrivals);

	  template <class ArrType>
	  void
	  cleanup_arrivals(const typename ArrType::iterator&	frame,
			   ArrType&				arrivals);
	};

      } // End of namespace internals.


      template <class InputIterator1,
		class InputIterator2,
		class Bottom = tool::bottom>
      struct intersector:
	tt_inherit(public tool::iterable, intersector<InputIterator1,
						      InputIterator2,
						      Bottom>)
      {
	intersector(const InputIterator1& first1, const InputIterator1& last1,
		    const InputIterator2& first2, const InputIterator2& last2);

      private:
	typedef std::set<typename InputIterator1::value_type> arrivals1_type;
	typedef std::set<typename InputIterator2::value_type> arrivals2_type;

	arrivals1_type	arrivals1_;
	arrivals2_type	arrivals2_;

	InputIterator1	next1_;
	InputIterator1	last1_;
	InputIterator2	next2_;
	InputIterator2	last2_;

	friend
	  class internals::intersector_iterator<InputIterator1,
						InputIterator2,
						Bottom,
						tool::bottom>;
      };

    } // End of namespace wifi::frame::filter.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "intersector.hxx"

#endif // ! WIFI_FRAME_FILTER_INTERSECTOR_HH_
