/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_SYNCHRONIZER_HH_
# define WIFI_FRAME_FILTER_SYNCHRONIZER_HH_

# include <boost/optional.hpp>
# include <gmpxx.h>

# include <trace-tools/tool/microseconds.hh>
# include <trace-tools/tool/iterator.hh>
# include <trace-tools/tool/iterable.hh>

# include "unique_synchronizer_fwd.hh"

namespace tool
{

  template <class I, class B1, class B2>
  struct types< wifi::frame::filter::internals::u_sync_iterator<I, B1, B2> >
  {
    typedef
      wifi::frame::filter::synchronized_unique_frame
      value_type;
  };

  template <class I, class B>
  struct types< wifi::frame::filter::unique_synchronizer<I, B> >
  {
    typedef
      wifi::frame::filter::internals::u_sync_iterator<I, B, bottom>
      iterator;
  };

} // End of namespace tool.

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      struct synchronized_unique_frame
      {
	typedef mpf_class			impl_type;
	typedef std::pair<impl_type, impl_type>	coefs_type;

	synchronized_unique_frame(const tool::microseconds&,
				  const tool::microseconds&,
				  const coefs_type&);

	bool	operator == (const synchronized_unique_frame&) const;
	bool	operator != (const synchronized_unique_frame&) const;

	tool::microseconds	t1;
	tool::microseconds	t2;
	coefs_type		coefs;
      };

      namespace internals
      {

	template <class I, class B, class Bottom = tool::bottom>
	struct u_sync_iterator:
	  tt_inherit(public tool::iterator, u_sync_iterator<I, B, Bottom>)
	{
	  typedef tt_get_exact(Bottom, u_sync_iterator<I, B, Bottom>)
								exact_type;
	  typedef tt_type(value_type, exact_type)		value_type;

	  u_sync_iterator(unique_synchronizer<I, B>&, bool);

 	  bool			equal(const exact_type&) const;
	  void			increment();
	  const value_type&	get() const;
	  const value_type*	get_ptr() const;

	private:
	  typedef boost::optional<value_type>	optional_value;

	  unique_synchronizer<I, B>*	i_;
	  unsigned			next_wpos_;
	  optional_value		v_;
	};

      } // End of namespace wifi::frame::filter::internals.

      template <class InputIterator, class Bottom = tool::bottom>
      struct unique_synchronizer:
	tt_inherit(public tool::iterable,
		   unique_synchronizer<InputIterator, Bottom>)
      {
	unique_synchronizer(const InputIterator& first,
			    const InputIterator& last);

      private:
	InputIterator	next_;
	InputIterator	last_;

	friend
	  class internals::u_sync_iterator<InputIterator, Bottom, tool::bottom>;
      };

    } // End of namespace wifi::frame::filter.
  } // End of namespace wifi::frame.
} // End of namespace wifi.

# include "unique_synchronizer.hxx"

#endif // ! WIFI_FRAME_FILTER_SYNCHRONIZER_HH_
