/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_ID_HELPER_HOOKS_HXX_
# define WIFI_FRAME_ID_HELPER_HOOKS_HXX_

# include "id_helper_hooks.hh"

namespace wifi
{

  namespace frame
  {

    namespace internals
    {

      inline
      id_helper_hooks::id_helper_hooks():
	timestamp_ (0),
	addrs_ (0)
      {
      }

      inline
      const timestamp*
      id_helper_hooks::timestamp_ptr() const
      {
	return timestamp_;
      }

      inline
      const addr*
      id_helper_hooks::addr_ptr(unsigned i) const
      {
	return addrs_ ? addrs_ + i : 0;
      }

      inline
      void
      id_helper_hooks::beacon_hook(const mgt::header* h, size_t caplen)
      {
	if (caplen < sizeof (*h) + sizeof (*timestamp_))
	  return;

	timestamp_ = reinterpret_cast<const timestamp*> (h->body);
	addrs_ = h->addrs;
      }

    } // End of namespace wifi::frame::internals.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_ID_HELPER_HOOKS_HXX_

