/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_MACTIME_TRACKER_HXX_
# define WIFI_MACTIME_TRACKER_HXX_

# include "mactime_tracker.hh"

namespace wifi
{

  inline
  mactime_tracker::mactime_tracker():
    mactime_ (0),
    wraps_ (0)
  {
  }

  inline
  void
  mactime_tracker::tick(mactime m)
  {
    if (m < mactime_)
      ++wraps_;
    mactime_ = m;
  }

  inline
  void
  mactime_tracker::tick(const prism::header* h, size_t caplen)
  {
    if (caplen < sizeof (prism::header))
      return;

    tick(h->mactime.data);
  }

  inline
  mactime_tracker::operator tool::microseconds () const
  {
    return tool::microseconds(wraps_, mactime_);
  }

} // End of namespace wifi.

#endif // ! WIFI_MACTIME_TRACKER_HXX_
