/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <cstdlib>
#include <vector>
#include <iostream>

#include <trace-tools/tool/window.hh>

template <unsigned WSize, unsigned Step>
bool
check(const std::vector<int>& v, bool debug)
{
  typedef typename std::vector<int>::const_iterator	vec_iterator;
  typedef tool::window_maker<vec_iterator, WSize, Step>	window_maker;
  typedef typename window_maker::iterator		window;

  window_maker	wm (v.begin(), v.end());
  unsigned	i = 0;

  if (debug)
    std::cerr << "** window size = " << WSize
	      << ", step = " << Step << std::endl;

  for (window w = wm.begin(); w != wm.end(); ++w)
    {
      if (debug)
	std::cerr << "* i = " << i << std::endl;

      unsigned j = 0;
      for (std::deque<int>::const_iterator wi = w->begin();
	   wi != w->end();
	   ++wi)
	{
	  if (debug)
	    std::cerr << "- got " << *wi
		      << " (expecting " << v[i + j] << ')' << std::endl;
	  if (*wi != v[i + j] or *wi != w[j] or *wi != (*w)[j])
	    return false;
	  ++j;
	}
      if (debug)
	std::cerr << "- end of window, j = " << i << std::endl;
      if (j != WSize and i + j < v.size())
	abort();
      i += Step;
    }
  if (debug)
    std::cerr << "end, i = " << i << std::endl;
  if (i + WSize < v.size())
    return false;
  return true;
}

int
main()
{
  const bool	debug = getenv("DEBUG") != 0;

  int			v_[] = { 1, 2, 3, 4 };
  std::vector<int>	v (v_, v_ + sizeof (v_) / sizeof (*v_));

  if (not check<1, 1>(v, debug)) abort();
  if (not check<2, 1>(v, debug)) abort();
  if (not check<3, 1>(v, debug)) abort();
  if (not check<4, 1>(v, debug)) abort();
  if (not check<5, 1>(v, debug)) abort();

  if (not check<2, 2>(v, debug)) abort();
  if (not check<3, 2>(v, debug)) abort();
  if (not check<4, 2>(v, debug)) abort();
  if (not check<5, 2>(v, debug)) abort();

  if (not check<3, 3>(v, debug)) abort();
  if (not check<4, 3>(v, debug)) abort();
  if (not check<5, 3>(v, debug)) abort();

  if (not check<4, 4>(v, debug)) abort();
  if (not check<5, 4>(v, debug)) abort();

  if (not check<5, 5>(v, debug)) abort();
  if (not check<6, 5>(v, debug)) abort();
}
